import { Controller, Get, Post, Body, Patch, Param, Delete, ParseIntPipe, HttpCode, UseGuards, Req, Query, NotFoundException } from '@nestjs/common'
import { ChallengeActionsService } from './challenge-actions.service'
import { AuthenticatedRequest, JwtAuthGuard } from 'src/auth/jwt-auth.guard'
import { ApiBearerAuth, ApiNoContentResponse } from '@nestjs/swagger'
import { ChallengeActionEntity } from './entities/challenge-action.entity'
import { CreateChallengeActionDto } from './dto/create-challenge-action.dto'
import { ApiOkResponsePaginated, paginateOutput } from 'src/common/utils/pagination.utils'
import { QueryPaginationDto } from 'src/common/dto/pagination-query.dto'
import { PaginateOutputDto } from 'src/common/dto/pagination-output.dto'
import { UpdateChallengeActionDto } from './dto/update-challenge-action.dto'
import { FilterChallengeActionsDto } from './dto/filter-challenge-action.dto'
import { EndChallengeActionDto } from './dto/end-challenge-action.dto'

@Controller('challenge-actions')
export class ChallengeActionsController {
  constructor(private readonly challengeActionsService: ChallengeActionsService) {}

  /**
   * Création d'une action de défi
   * 
   * @throws {400} Erreurs dans le formulaire de création
   * @throws {401} Non authentifié⋅e
   */
  @Post()
  @HttpCode(201)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  async create(@Req() request: AuthenticatedRequest, @Body() createChallengeActionDto: CreateChallengeActionDto): Promise<ChallengeActionEntity> {
    const challenge = await this.challengeActionsService.create(request.user, createChallengeActionDto)
    return new ChallengeActionEntity(challenge)
  }

  /**
   * Recherche d'actions de défi
   * 
   * @throws {401} Non authentifié⋅e
   */
  @Get()
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponsePaginated(ChallengeActionEntity)
  async findAll(@Query() queryPagination: QueryPaginationDto, @Query() filterChallengeActions: FilterChallengeActionsDto): Promise<PaginateOutputDto<ChallengeActionEntity>> {
    const [challengeActions, total] = await this.challengeActionsService.findAll(queryPagination, filterChallengeActions)
    return paginateOutput<ChallengeActionEntity>(challengeActions.map(challengeAction => new ChallengeActionEntity(challengeAction)), total, queryPagination)
  }

  /**
   * Recherche d'une action de défi par identifiant
   * 
   * @throws {401} Non authentifié⋅e
   * @throws {404} Action de défi non trouvée
   */
  @Get(':id')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  async findOne(@Param('id', ParseIntPipe) id: number): Promise<ChallengeActionEntity> {
    const challenge = await this.challengeActionsService.findOne(id)
    if (!challenge)
      throw new NotFoundException(`Défi inexistant avec l'identifiant ${id}`)
    return new ChallengeActionEntity(challenge)
  }

  /**
   * Modification d'une action de défi par identifiant
   * 
   * @throws {400} Erreurs dans le formulaire de modification
   * @throws {401} Non authentifié⋅e
   * @throws {404} Action de défi non trouvée
   */
  @Patch(':id')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  async update(@Param('id', ParseIntPipe) id: number, @Body() updateChallengeActionDto: UpdateChallengeActionDto): Promise<ChallengeActionEntity> {
    return await this.challengeActionsService.update(id, updateChallengeActionDto)
  }

  /**
   * Suppression d'une action de défi par identifiant
   * 
   * @throws {401} Non authentifié⋅e
   * @throws {404} Action de défi non trouvée
   */
  @Delete(':id')
  @HttpCode(204)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiNoContentResponse({ description: "Action de défi supprimée avec succès" })
  async remove(@Param('id', ParseIntPipe) id: number): Promise<void> {
    await this.challengeActionsService.remove(id)
  }

  /**
   * Terminer l'action de défi en cours
   * 
   * @throws {401} Non authentifié⋅e
   * @throws {409} Aucun défi à terminer n'est en cours
   */
  @Post('/end-current')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  async endCurrent(@Req() request: AuthenticatedRequest, @Body() { success }: EndChallengeActionDto): Promise<ChallengeActionEntity> {
    const challengeAction = await this.challengeActionsService.endCurrentChallenge(request.user, success)
    return new ChallengeActionEntity(challengeAction)
  }
}
