import { Controller, Get, Post, Body, Patch, Param, Delete, ParseIntPipe, HttpCode, UseGuards, Req, Query, NotFoundException } from '@nestjs/common'
import { ChallengesService } from './challenges.service'
import { CreateChallengeDto } from './dto/create-challenge.dto'
import { UpdateChallengeDto } from './dto/update-challenge.dto'
import { AuthenticatedRequest, JwtAuthGuard } from 'src/auth/jwt-auth.guard'
import { ApiBearerAuth, ApiNoContentResponse } from '@nestjs/swagger'
import { ChallengeEntity } from './entities/challenge.entity'
import { ApiOkResponsePaginated, paginateOutput } from 'src/common/utils/pagination.utils'
import { QueryPaginationDto } from 'src/common/dto/pagination-query.dto'
import { PaginateOutputDto } from 'src/common/dto/pagination-output.dto'

@Controller('challenges')
export class ChallengesController {
  constructor(private readonly challengesService: ChallengesService) {}

  /**
   * Création d'un défi
   * 
   * @throws {400} Erreurs dans le formulaire de création
   * @throws {401} Non authentifié⋅e
   */
  @Post()
  @HttpCode(201)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  async create(@Body() createChallengeDto: CreateChallengeDto): Promise<ChallengeEntity> {
    const challenge = await this.challengesService.create(createChallengeDto)
    return new ChallengeEntity(challenge)
  }

  /**
   * Recherche de défis
   * 
   * @throws {401} Non authentifié⋅e
   */
  @Get()
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponsePaginated(ChallengeEntity)
  async findAll(@Query() queryPagination?: QueryPaginationDto): Promise<PaginateOutputDto<ChallengeEntity>> {
    const [challenges, total] = await this.challengesService.findAll(queryPagination)
    return paginateOutput<ChallengeEntity>(challenges.map(challenge => new ChallengeEntity(challenge)), total, queryPagination)
  }

  /**
   * Recherche d'un défi par identifiant
   * 
   * @throws {401} Non authentifié⋅e
   * @throws {404} Défi non trouvé
   */
  @Get(':id')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  async findOne(@Param('id', ParseIntPipe) id: number): Promise<ChallengeEntity> {
    const challenge = await this.challengesService.findOne(id)
    if (!challenge)
      throw new NotFoundException(`Défi inexistant avec l'identifiant ${id}`)
    return new ChallengeEntity(challenge)
  }

  /**
   * Modification d'un défi par identifiant
   * 
   * @throws {400} Erreurs dans le formulaire de modification
   * @throws {401} Non authentifié⋅e
   * @throws {404} Défi non trouvé
   */
  @Patch(':id')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  async update(@Param('id', ParseIntPipe) id: number, @Body() updateChallengeDto: UpdateChallengeDto) {
    return await this.challengesService.update(id, updateChallengeDto)
  }

  /**
   * Suppression d'un défi par identifiant
   * 
   * @throws {401} Non authentifié⋅e
   * @throws {404} Défi non trouvé
   */
  @Delete(':id')
  @HttpCode(204)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiNoContentResponse({ description: "Le défi a bien été supprimé" })
  async remove(@Param('id', ParseIntPipe) id: number) {
    await this.challengesService.remove(id)
  }

  /**
   * Tirage d'un nouveau défi aléatoire
   * 
   * @remarks Aucun défi ne doit être en cours.
   * 
   * @throws {401} Non authentifié⋅e
   * @throws {404} Plus aucun défi n'est disponible
   * @throws {409} Un défi est déjà en cours d'accomplissement, ou bien vous n'êtes pas en course
   */
  @Post('/draw-random')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  async drawRandom(@Req() request: AuthenticatedRequest): Promise<ChallengeEntity> {
    const challenge = await this.challengesService.drawRandom(request.user)
    return new ChallengeEntity(challenge)
  }
}
