import { BadRequestException, ConflictException, Injectable, NotFoundException } from '@nestjs/common'
import { CreateChallengeDto } from './dto/create-challenge.dto'
import { UpdateChallengeDto } from './dto/update-challenge.dto'
import { Challenge, Player } from '@prisma/client'
import { PrismaService } from 'src/prisma/prisma.service'
import { QueryPaginationDto } from 'src/common/dto/pagination-query.dto'
import { paginate } from 'src/common/utils/pagination.utils'
import { ChallengeEntity } from './entities/challenge.entity'

@Injectable()
export class ChallengesService {
  constructor(private prisma: PrismaService) { }

  async create(createChallengeDto: CreateChallengeDto): Promise<Challenge> {
    const data = { ...createChallengeDto }
    if (await this.prisma.challenge.findFirst({ where: { title: createChallengeDto.title } }))
      throw new BadRequestException(`Un défi existe déjà avec pour titre « ${createChallengeDto.title} ».`)
    return await this.prisma.challenge.create({ data: data })
  }

  async findAll(queryPagination?: QueryPaginationDto): Promise<[Challenge[], number]> {
    return [
      await this.prisma.challenge.findMany({
        ...paginate(queryPagination),
        include: {
          action: true,
        }
      }),
      await this.prisma.challenge.count(),
    ]
  }

  async findOne(id: number): Promise<Challenge> {
    return await this.prisma.challenge.findUnique({
      where: { id },
      include: {
        action: true,
      }
    })
  }

  async update(id: number, updateChallengeDto: UpdateChallengeDto): Promise<Challenge> {
    if (!await this.findOne(id))
      throw new NotFoundException(`Aucun défi n'existe avec l'identifiant ${id}`)
    if (await this.prisma.challenge.findFirst({ where: { title: updateChallengeDto.title, id: { not: id } } }))
      throw new BadRequestException(`Un défi existe déjà avec pour titre « ${updateChallengeDto.title} ».`)
    return await this.prisma.challenge.update({
      where: { id },
      data: updateChallengeDto,
      include: {
        action: true,
      }
    })
  }

  async remove(id: number): Promise<Challenge> {
    if (!await this.findOne(id))
      throw new NotFoundException(`Aucun défi n'existe avec l'identifiant ${id}`)
    return await this.prisma.challenge.delete({
      where: { id },
      include: {
        action: true,
      }
    })
  }

  async drawRandom(player: Player): Promise<ChallengeEntity> {
    const game = await this.prisma.game.findUnique({ where: { id: 1 }, include: { currentRun: true } })
    if (game.currentRun?.runnerId !== player.id)
      throw new ConflictException("Vous n'êtes pas en course, ce n'est pas à vous de tirer un défi.")
    if (player.activeChallengeId)
      throw new ConflictException("Un défi est déjà en cours d'accomplissement")
    const remaningChallenges = await this.prisma.challenge.count({
      where: {
        action: null,
      }
    })
    const challenge = await this.prisma.challenge.findFirst({
      where: {
        action: null,
      },
      take: 1,
      skip: Math.random() * remaningChallenges,
    })
    if (!challenge)
      throw new NotFoundException("Plus aucun défi disponible")
    const challengeEntity: ChallengeEntity = new ChallengeEntity(challenge)
    const action = await this.prisma.challengeAction.create({
      data: {
        playerId: player.id,
        challengeId: challenge.id,
        runId: game.currentRunId,
        success: false,
      }
    })
    await this.prisma.player.update({
      where: { id: player.id },
      data: { activeChallengeId: action.id },
    })
    challengeEntity.action = action
    return challengeEntity
  }
}
