import { ConflictException, Injectable } from '@nestjs/common'
import { Game, MoneyUpdate, MoneyUpdateType } from '@prisma/client'
import { PrismaService } from 'src/prisma/prisma.service'
import { RepairGame } from './dto/repair-game.dto'
import { Constants } from 'src/common/utils/constants.utils'

@Injectable()
export class GameService {
  constructor (private prisma: PrismaService) {}

  async find() {
    return await this.prisma.game.findUnique({ where: { id: 1 }, include: { currentRun: true } })
  }

  async start(): Promise<Game> {
    const game = await this.find()
    if (game.started)
      throw new ConflictException("La partie a déjà démarré.")
    const players = await this.prisma.player.findMany()
    const alreadyStarted = game.currentRunId !== null
    let run
    if (!alreadyStarted) {
      for (const player of players) {
        await this.prisma.moneyUpdate.create({
          data: {
            playerId: player.id,
            amount: Constants.INITIAL_MONEY,
            reason: MoneyUpdateType.START,
          }
        })
      }
      const runnerId = players[Math.floor(players.length * Math.random())].id
      run = await this.prisma.playerRun.create({
        data: {
          gameId: game.id,
          runnerId: runnerId,
        }
      })
    }
    else {
      run = await this.prisma.playerRun.update({
        where: { id: game.currentRunId },
        data: { end: null },
      })
    }
    return await this.prisma.game.update({
      where: { id: 1 },
      data: {
        started: true,
        currentRunId: run.id,
      },
    })
  }

  async switchRunningPlayer(): Promise<Game> {
    const game = await this.find()
    if (!game.started)
      throw new ConflictException("La partie n'a pas encore démarré.")

    // Clôture de l'éventuel défi en cours, qui n'a alors pas été réussi
    const currentRunner = await this.prisma.player.findUnique({ where: { id: game.currentRun.runnerId } })
    if (currentRunner.activeChallengeId) {
      await this.prisma.challengeAction.update({
        where: { id: currentRunner.activeChallengeId },
        data: { success: false },
      })
      await this.prisma.player.update({
        where: { id: currentRunner.id },
        data: { activeChallengeId: null },
      })
    }

    await this.prisma.playerRun.update({
      where: { id: game.currentRunId },
      data: { end: new Date() },
    })

    const newRunnerId = game.currentRun.runnerId == 1 ? 2 : 1
    const firstRun = await this.prisma.playerRun.findFirst({ where: { runnerId: newRunnerId } }) === null
    const newRun = await this.prisma.playerRun.create({
      data: {
        gameId: game.id,
        runnerId: newRunnerId,
      }
    })
    if (!firstRun) {
      // Si c'est une nouvelle course (pas la première), on accorde un bonus de points de départ
      await this.prisma.moneyUpdate.create({
        data: {
          playerId: newRunnerId,
          amount: Constants.NEW_RUN_MONEY,
          reason: MoneyUpdateType.NEW_RUN,
          runId: newRun.id,
        }
      })
    }
    return await this.prisma.game.update({
      where: { id: game.id },
      data: { currentRunId: newRun.id },
    })
  }

  async stop(): Promise<Game> {
    const game = await this.find()
    if (!game.started)
      throw new ConflictException("La partie n'a pas encore démarré.")
    await this.prisma.playerRun.update({
      where: { id: game.currentRunId },
      data: { end: new Date() },
    })
    return await this.prisma.game.update({
      where: { id: 1 },
      data: { started: false },
    })
  }

  async reset(): Promise<Game> {
    await this.prisma.game.update({
      where: { id: 1 },
      data: {
        started: false,
        currentRunId: null,
      },
    })
    await this.prisma.player.updateMany({ data: { money: 0 } })
    await this.prisma.moneyUpdate.deleteMany()
    await this.prisma.challengeAction.deleteMany()
    await this.prisma.trainTrip.deleteMany()
    await this.prisma.playerRun.deleteMany()
    await this.prisma.geolocation.deleteMany()
    return await this.find()
  }

  async repair(): Promise<RepairGame> {
    const added: MoneyUpdate[] = []
    const modified: MoneyUpdate[] = []
    const deleted: MoneyUpdate[] = []
    const trains = await this.prisma.trainTrip.findMany({ include: { moneyUpdate: true } })
    for (const train of trains) {
      if (!train.moneyUpdate) {
        const trainMoneyUpdate = await this.prisma.moneyUpdate.create({
          data: {
            playerId: train.playerId,
            amount: -Constants.PRICE_PER_KILOMETER * Math.ceil(train.distance / 1000),
            reason: MoneyUpdateType.BUY_TRAIN,
            tripId: train.id,
          }
        })
        added.push(trainMoneyUpdate)
      }
      else {
        const moneyUpdate = train.moneyUpdate
        const cost = Constants.PRICE_PER_KILOMETER * Math.ceil(train.distance / 1000)
        if (moneyUpdate.amount != -cost) {
          const modifiedMoneyUpdate = await this.prisma.moneyUpdate.update({
            where: { id: moneyUpdate.id },
            data: { amount: -cost },
          })
          modified.push(modifiedMoneyUpdate)
        }
      }
    }
    const orpanTrainMoneyUpdates = await this.prisma.moneyUpdate.findMany({ where: { reason: MoneyUpdateType.BUY_TRAIN, tripId: null } })
    await this.prisma.moneyUpdate.deleteMany({ where: { reason: MoneyUpdateType.BUY_TRAIN, tripId: null } })
    deleted.push(...orpanTrainMoneyUpdates)

    const challengeActions = await this.prisma.challengeAction.findMany({ include: { moneyUpdate: true, challenge: true, activePlayer: true } })
    for (const challengeAction of challengeActions) {
      if (challengeAction.success && !challengeAction.moneyUpdate) {
        const challengeMoneyUpdate = await this.prisma.moneyUpdate.create({
          data: {
            playerId: challengeAction.playerId,
            amount: challengeAction.challenge.reward,
            reason: MoneyUpdateType.CHALLENGE,
            actionId: challengeAction.id,
          }
        })
        added.push(challengeMoneyUpdate)
      }
      else if (challengeAction.success && challengeAction.moneyUpdate.amount !== challengeAction.challenge.reward) {
        const modifiedMoneyUpdate = await this.prisma.moneyUpdate.update({
          where: { id: challengeAction.moneyUpdate.id },
          data: { amount: challengeAction.challenge.reward },
        })
        modified.push(modifiedMoneyUpdate)
      }
      else if (!challengeAction.success && !challengeAction.activePlayer && !challengeAction.moneyUpdate) {
        const challengeMoneyUpdate = await this.prisma.moneyUpdate.create({
          data: {
            playerId: challengeAction.playerId,
            amount: 0,
            reason: MoneyUpdateType.CHALLENGE,
            actionId: challengeAction.id,
          }
        })
        added.push(challengeMoneyUpdate)
      }
      else if (!challengeAction.success && !challengeAction.activePlayer && challengeAction.moneyUpdate.amount !== 0) {
        const modifiedMoneyUpdate = await this.prisma.moneyUpdate.update({
          where: { id: challengeAction.moneyUpdate.id },
          data: { amount: 0 },
        })
        modified.push(modifiedMoneyUpdate)
      }
      else if (!challengeAction.success && challengeAction.activePlayer && challengeAction.moneyUpdate) {
        deleted.push(challengeAction.moneyUpdate)
        await this.prisma.moneyUpdate.delete({ where: { id: challengeAction.moneyUpdate.id } })
      }
    }

    const orpanChallengeMoneyUpdates = await this.prisma.moneyUpdate.findMany({ where: { reason: MoneyUpdateType.CHALLENGE, actionId: null } })
    await this.prisma.moneyUpdate.deleteMany({ where: { reason: MoneyUpdateType.CHALLENGE, actionId: null } })
    deleted.push(...orpanChallengeMoneyUpdates)

    return { added: added, modified: modified, deleted: deleted }
  }
}
