import { Injectable, NotFoundException } from '@nestjs/common'
import { CreateGeolocationDto } from './dto/create-geolocation.dto'
import { PrismaService } from 'src/prisma/prisma.service'
import { Geolocation, Player, Prisma } from '@prisma/client'
import { QueryPaginationDto } from 'src/common/dto/pagination-query.dto'
import { paginate } from 'src/common/utils/pagination.utils'
import { PlayerFilterDto } from 'src/common/dto/player_filter.dto'

@Injectable()
export class GeolocationsService {
  constructor(private prisma: PrismaService) { }

  async create(authenticatedPlayer: Player, createGeolocationDto: CreateGeolocationDto): Promise<Geolocation> {
    const data = { ...createGeolocationDto, playerId: authenticatedPlayer.id }
    return await this.prisma.geolocation.create({ data: data })
  }

  async findAll(queryPagination?: QueryPaginationDto, playerFilter?: PlayerFilterDto): Promise<[Geolocation[], number]> {
    const filter = {
      where: (playerFilter?.playerId ? { playerId: playerFilter.playerId } : {}),
      orderBy: { timestamp: Prisma.SortOrder.desc },
    }
    return [
      await this.prisma.geolocation.findMany({
        ...filter,
        ...paginate(queryPagination),
      }),
      await this.prisma.geolocation.count(filter),
    ]
  }

  async findOne(id: number): Promise<Geolocation> {
    return await this.prisma.geolocation.findUnique({ where: { id } })
  }

  async findLastLocations(): Promise<Geolocation[]> {
    const players = await this.prisma.player.findMany()
    const lastGeolocations = []
    for (const player of players) {
      const lastGeolocationPlayer = await this.prisma.geolocation.findFirst({
        where: { playerId: player.id },
        orderBy: { timestamp: 'desc' },
        take: 1,
      })
      if (lastGeolocationPlayer)
        lastGeolocations.push(lastGeolocationPlayer)
    }
    return lastGeolocations
  }

  async remove(id: number): Promise<Geolocation> {
    if (!await this.findOne(id))
      throw new NotFoundException(`Aucune géolocalisation n'existe avec l'identifiant ${id}`)
    return await this.prisma.geolocation.delete({ where: { id } })
  }
}
