import { Body, Controller, Get, HttpCode, NotFoundException, Param, ParseIntPipe, Patch, Query, Req, UseGuards } from '@nestjs/common'
import { PlayersService } from './players.service'
import { ApiBearerAuth, ApiNoContentResponse } from '@nestjs/swagger'
import { PlayerEntity } from './entities/player.entity'
import { AuthenticatedRequest, JwtAuthGuard } from 'src/auth/jwt-auth.guard'
import { UpdatePasswordDto } from './dto/player_password.dto'
import { QueryPaginationDto } from 'src/common/dto/pagination-query.dto'
import { ApiOkResponsePaginated, paginateOutput } from 'src/common/utils/pagination.utils'
import { PaginateOutputDto } from 'src/common/dto/pagination-output.dto'

@Controller('players')
export class PlayersController {
  constructor(private readonly playersService: PlayersService) {}

  /**
   * Récupération de toustes les joueur⋅ses
   * 
   * @throws {401} Non authentifié⋅e
   */
  @Get()
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponsePaginated(PlayerEntity)
  async findAll(@Query() queryPagination?: QueryPaginationDto): Promise<PaginateOutputDto<PlayerEntity>> {
    const [players, total] = await this.playersService.findAll(queryPagination)
    return paginateOutput<PlayerEntity>(players.map(player => new PlayerEntity(player)), total, queryPagination)
  }

  /**
   * Récupération d'un⋅e joueur⋅se par son identifiant
   * 
   * @throws {401} Non authentifié⋅e
   * @throws {404} Joueur⋅se non trouvé⋅e
   */
  @Get(':id')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  async findOne(@Param('id', ParseIntPipe) id: number): Promise<PlayerEntity> {
    const player = await this.playersService.findOne(id)
    if (!player)
      throw new NotFoundException(`Læ joueur⋅se avec l'identifiant ${id} n'existe pas`)
    return new PlayerEntity(player)
  }

  /**
   * Modification du mot de passe
   * 
   * @throws {400} Mot de passe invalide
   * @throws {401} Non authentifié⋅e
   */
  @Patch('/update-password')
  @HttpCode(204)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiNoContentResponse({ description: "Le mot de passe a bien été modifié." })
  async updatePassword(@Req() request: AuthenticatedRequest, @Body() body: UpdatePasswordDto): Promise<void> {
    await this.playersService.updatePassword(request.user, body)
  }
}
